@@ -2,8 +2,33 @@ import tensorflow as tf
 from tensorflow import keras
 
 class GeneralizedLogistic(tf.keras.Model):
+    '''
+    Trainable generalized logistic growth model.
 
-    def __init__(self, units=1, input_dim=1):
+    Also called "Richard's curve".
+
+    Attributes
+    -------
+    K : float
+        Upper asymptote.
+    A : float
+        Lower asymptote.
+    B : float
+        Growth rate.
+    nu : float
+        Parameter that affects asymptotic growth.
+    Q : float
+        Parameter related initial value of dependent variable.
+    C : float
+        Parameter related to upper asymptote.
+
+    Method
+    -------
+    call(inputs)
+        Predicts dependent variable from input.
+    '''
+
+    def __init__(self, units=1, input_dim=1, **kwargs):
         super().__init__()
         self.K = self.add_weight(
             shape=(units, input_dim),
@@ -23,7 +48,10 @@ class GeneralizedLogistic(tf.keras.Model):
             )
         self.B=self.add_weight(
             shape=(units, input_dim),
-            initializer='random_normal',
+            initializer=tf.keras.initializers.RandomUniform(
+                minval=0.,
+                maxval=2
+                ),
             trainable=True
             )
         self.nu=self.add_weight(
@@ -57,6 +85,12 @@ class GeneralizedLogistic(tf.keras.Model):
             )
 
     def call(self, inputs):
+        '''
+        Parameters
+        ----------
+        inputs : array-like[float]
+            Input array of data.
+        '''
         result = inputs - self.M
         result = self.B * result
         result = -result
@@ -73,7 +107,7 @@ class Gaussian(tf.keras.Model):
     https://en.wikipedia.org/wiki/Gaussian_function
     '''
 
-    def __init__(self, units=1, input_dim=1):
+    def __init__(self, units=1, input_dim=1, **kwargs):
         super().__init__()
         self.a=self.add_weight(
             shape=(units, input_dim),
@@ -104,7 +138,7 @@ class Gaussian(tf.keras.Model):
 
 class Gompertz(tf.keras.Model):
 
-    def __init__(self, units=1, input_dim=1):
+    def __init__(self, units=1, input_dim=1, **kwargs):
         super().__init__()
         self.a=self.add_weight(
             shape=(units, input_dim),
@@ -156,38 +190,6 @@ class HyperbolasticTypeIII(tf.keras.Model):
     https://en.wikipedia.org/wiki/Hyperbolastic_functions#Function_H3
     '''
 
-class VanGenuchtenGupta(tf.keras.Model):
-    '''
-    https://en.wikipedia.org/wiki/Van_Genuchten%E2%80%93Gupta_model
-    '''
-
-    def __init__(self, units=1, input_dim=1):
-        super().__init__()
-        self.Emax=self.add_weight(
-            shape=(units, input_dim),
-            initializer='random_uniform',
-            trainable=True
-            )
-        self.EC50=self.add_weight(
-            shape=(units, input_dim),
-            initializer='random_uniform',
-            trainable=True
-            )
-        self.n=self.add_weight(
-            shape=(units, input_dim),
-            initializer='random_uniform',
-            trainable=True
-            )
-
-
-    def call(self, inputs):
-        result = self.EC50 / inputs
-        result = tf.math.pow(result, self.n)
-        result = 1.0 + result
-        result = 1 / result
-        result = self.Emax * result
-        return result
-
 class Linear(tf.keras.Model):
     '''
     Example
@@ -197,7 +199,7 @@ class Linear(tf.keras.Model):
     >>> y = model(x)
     '''
 
-    def __init__(self, units=1, input_dim=1):
+    def __init__(self, units=1, input_dim=1, **kwargs):
         super().__init__()
         self.w = self.add_weight(
             shape=(units, input_dim),
@@ -218,7 +220,7 @@ class MaasHoffman(tf.keras.Model):
     https://en.wikipedia.org/wiki/Maas%E2%80%93Hoffman_model
     '''
 
-    def __init__(self, units=1, input_dim=1):
+    def __init__(self, units=1, input_dim=1, **kwargs):
         super().__init__()
         self.a = self.add_weight(
             shape=(units, input_dim),
@@ -248,12 +250,44 @@ class MaasHoffman(tf.keras.Model):
             self.c * tf.ones(inputs.shape)
             )
 
+class Signum(tf.keras.Model):
+
+    def __init__(self, units=1, input_dim=1, **kwargs):
+        super().__init__()
+        self.a=self.add_weight(
+                shape=(units, input_dim),
+                initializer='random_normal',
+                trainable=True
+                )
+        self.b=self.add_weight(
+                shape=(units, input_dim),
+                initializer='random_normal',
+                trainable=True
+                )
+        self.c=self.add_weight(
+                shape=(units, input_dim),
+                initializer='random_normal',
+                trainable=True
+                )
+        self.d=self.add_weight(
+                shape=(units, input_dim),
+                initializer='random_normal',
+                trainable=True
+                )
+    def call(self, inputs):
+        result = self.b * inputs
+        result = result + self.c
+        result = tf.math.sign(result)
+        result = self.a * result
+        result = result + self.d
+        return result
+
 class SuperGaussian(tf.keras.Model):
     '''
     https://en.wikipedia.org/wiki/Gaussian_function#Higher-order_Gaussian_or_super-Gaussian_function
     '''
 
-    def __init__(self, units=1, input_dim=1):
+    def __init__(self, units=1, input_dim=1, **kwargs):
         super().__init__()
         self.a=self.add_weight(
             shape=(units, input_dim),
@@ -288,9 +322,41 @@ class SuperGaussian(tf.keras.Model):
         result = self.a * result
         return result
 
+class VanGenuchtenGupta(tf.keras.Model):
+    '''
+    https://en.wikipedia.org/wiki/Van_Genuchten%E2%80%93Gupta_model
+    '''
+
+    def __init__(self, units=1, input_dim=1, **kwargs):
+        super().__init__()
+        self.Emax=self.add_weight(
+            shape=(units, input_dim),
+            initializer='random_uniform',
+            trainable=True
+            )
+        self.EC50=self.add_weight(
+            shape=(units, input_dim),
+            initializer='random_uniform',
+            trainable=True
+            )
+        self.n=self.add_weight(
+            shape=(units, input_dim),
+            initializer='random_uniform',
+            trainable=True
+            )
+
+
+    def call(self, inputs):
+        result = self.EC50 / inputs
+        result = tf.math.pow(result, self.n)
+        result = 1.0 + result
+        result = 1 / result
+        result = self.Emax * result
+        return result
+
 class Verhulst(tf.keras.Model):
 
-    def __init__(self, units=1, input_dim=1):
+    def __init__(self, units=1, input_dim=1, **kwargs):
         super().__init__()
         self.k = self.add_weight(
             shape=(units, input_dim),
@@ -325,7 +391,7 @@ class Verhulst(tf.keras.Model):
 
 class VonBertalanffy(tf.keras.Model):
 
-    def __init__(self, units=1, input_dim=1):
+    def __init__(self, units=1, input_dim=1, **kwargs):
         super().__init__()
         self.L=self.add_weight(
             shape=(units, input_dim),
@@ -351,4 +417,46 @@ class VonBertalanffy(tf.keras.Model):
         result = self.L * result
         return result
 
+class SEM(tf.keras.Model):
+    '''
+    A toy structural equation model.
+    '''
 
+    def __init__(self, units=1, input_dim=1, **kwargs):
+        super().__init__()
+        self.model1 = Gaussian()
+        self.model2 = Gaussian()
+
+    def call(self, inputs):
+        return tf.concat((self.model1(x), self.model2(x)), 1)
+
+if __name__ == '__main__':
+    import matplotlib.pyplot as plt
+
+    # Constuct/Import data
+    x = tf.abs(tf.random.uniform((100000,1), 0, 10))
+    y = 500 / (1 + (500-50)/50 * tf.exp(-0.5 * x))
+
+    # Construct model
+    model = Signum()
+    optimizer = tf.keras.optimizers.Nadam(learning_rate=0.5)
+    loss = tf.keras.losses.MeanSquaredError()
+    model.compile(optimizer=optimizer, loss=loss)
+
+
+##    # Train model
+##    history = model.fit(x, y, epochs=20, batch_size=1000)
+##
+##    # Inspect results
+##    print(model.weights)
+##
+##    fig, axes = plt.subplots(2, 1)
+##    axes[0].scatter(x,y, alpha=0.5, s=1)
+##    axes[0].scatter(x, model(x), alpha=0.5, s=1)
+##    axes[0].set_ylabel('y')
+##    axes[0].set_xlabel('x')
+##
+##    axes[1].plot(history.history['loss'])
+##    axes[1].set_ylabel(loss.name)
+##    axes[1].set_xlabel('Epoch')
+##    plt.show()
